<?php
/*
 * Poplar Gedcom Viewer
 * Copyright (C) 2002-2003 Nicolas de Vulpian
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/*
 * Extensively rewritten 10/2003
 * changes Copyright 2003 by J.R.Grotjan
*/
?>
<?php

/**
* Pgv class handles everything related to the output.
* @author Nicolas de Vulpian
* @version 1.2
*/

class Pgv {

    /**
    * Contains current PGV's environment variables. It includes all configuration details
    *
    * environment['rootPath'] is the physical path to PHP Gedcom Viewer's root directory
    * / environment['rootUrl'] is the URL to PHP Gedcom Viewer's root directory
    * / environment['dbPrefix'] is the prefix to be used in front of each table created
    * @access public
    * @type array
    **/

    var $environment;

    /**
    * Contains informations related to the configuration of the genealogy beeing viewed
    *
    * @access public
    * @type array
    **/

    var $info;

    /**
    * Contains PGV settings.
    *
    * admin_contact is the name or email of the administrator of the genealogies
    *  / language is the default language, that will be set if the browser's language is not detected
    *  / default_theme is the theme used for the global index and for genealogies with no theme set
    *  / result_number is the number of results displayed per page in the search engine
    *  / debug_mode
    *  / logo_display
    * @access public
    * @type array
    **/

    var $settings;

    /**
    * Contains the privacy settings for the current genealogy
    *
    * @access public
    * @type array
    **/

    var $privacy;


    /**
    * Contains the data relative to the current state of the application
    *
    * @access public
    * @type array
    **/

    var $context;

    /**
    * Array of fragments used to create an A link
    *
    * @access private
    * @type array
    **/

    var $_aLink;

    /**
    * Contains the SQL table names of the genealogy beeing viewed
    *
    * @access private
    * @type array
    **/

    var $table;

    /**
    * Contains the list of enabled modules and their informations, in the page beeing viewed
    *
    * @access public
    * @type array
    **/

    var $modules;

    /**
    * Contains the information related to a selected module
    *
    * @access public
    * @type array
    **/

    var $module;

    /**
    * Constructor. Generates all the array used to run PHP Gedcom Viewer
    *
    * @access public
    * @type array
    * @returns void
    * @param $skipQueryDb if set to true, skips the database query. false by default.
    **/

    function pgv( $skipQueryDB = false ) {
        $this->_getEnvironment();
        $this->_getTables();
        if ( !$skipQueryDB ) {
            $this->_queryDB();
        }
        $this->_getContext();
    }

    /**
    * Generates the $info array
    *
    * @access private
    * @return void
    **/

    function _queryDB() {
        global $db;

        $infos = $db->GetAll( 'SELECT * FROM '.$this->table['info']
        .' WHERE genID=\''.$_GET['genID'].'\'' );
        if ( $infos === false ) die ( 'Failed to select from '
        .$this->table['info'].', class pgv(), method _queryDB()' );

        if ( !empty( $infos[0] ) ) {
            foreach( $infos[0] as $key=>$value ) {
                $this->infos[$key] = $value;
            }
        }

        $settings = $db->GetAll( 'SELECT * FROM '.$this->table['settings']
        .' WHERE sID = 1' );
        if ( $settings === false )
        die('Failed to load settings. Please run the <a href=\'.\install.php\'>install script</a> first!');

        foreach( $settings[0] as $key=>$value ) {
            $this->settings[$key] = $value;
        }

        unset($this->privacy);  //clear any prev settings
        $query = 'SELECT * FROM '.$this->moduletable('privacy')
        .' WHERE genID='.$_GET['genID'];
        $priv = $db->GetAll( $query );
        if ( count($priv[0]) ){
            foreach( $priv[0] as $key=>$value )
                $this->privacy[$key] = $value;
        }
    }

    /**
    * Generates the environment array
    *
    * @access private
    * @return void
    **
    **/

    function _getEnvironment() {
        global $env;

        $this->environment = $env;
    }

    /**
    * Generates the context array
    *
    * @access private
    * @return void
    **
    **/

    function _getContext() {
        $this->context['genName']        = $this->infos['table_name'];
        $this->context['genID']          = $this->infos['genID'];
        if ( !empty( $_GET['page'] ) ) {
            $this->context['page']       = ereg_replace( '[^[:alnum:]_]' , '' , $_GET['page'] ); // avoids including of sensitive file
        } else {
            $this->context['page']       = 'welcome';
        }
        $this->context['letter']         = $_GET['letter'];
        $this->context['advanced']       = $_GET['advanced'];
        $this->context['module']         = $_GET['module'];
        $this->context['idSurname']      = urldecode( $_GET['id_surname'] );
        $this->context['indID']          = $_GET['id'];
        $this->context['spouseID']       = $_GET['spouseID'];
        $this->context['ul']             = $_GET['ul'];

        $this->context['language']       = $this->_getLanguage();
        $this->context['isGen']          = !empty( $this->context['genID'] );
        $this->context['isGenHome']      = ( $this->context['isGen'] && !empty( $this->context['page'] ) );
        $this->context['isIndex']        = !$this->context['genID']; //inverse of isGen
        $this->context['adminContact']   = $this->settings['admin_contact'];

        $this->context['isModule']       = !empty( $this->context['module'] );
        $this->context['hasModuleTheme'] = file_exists( $this->environment['rootPath'].'/modules/'.$this->context['module'].'/module_theme.tpl' );
        $this->context['ownerName']      = $this->infos['owner_name'];
        $this->context['ownerEmail']     = $this->infos['owner_email'];
        $this->context['ownerWebsite']   = $this->infos['owner_website'];

        $this->context['displayLogo']    = $this->settings['logo_display'];

        $this->context['charset']        = 'iso-8859-1';
        $this->context['defaultTheme']   = $this->settings['default_theme'];
        $this->context['theme']          = $this->_getTheme();
    }

    /**
    * Generates the table array
    *
    * @access private
    * @return void
    **
    **/

    function _getTables() {
        $this->table['info']        = $this->environment['dbPrefix'].'_ged_info';
        $this->table['settings']    = $this->environment['dbPrefix'].'_settings';
        $this->table['modules']     = $this->environment['dbPrefix'].'_modules';
        $this->table['individuals'] = $this->environment['dbPrefix'].'_individuals';
        $this->table['families']    = $this->environment['dbPrefix'].'_families';
        $this->table['facts']       = $this->environment['dbPrefix'].'_facts';
        $this->table['comments']    = $this->environment['dbPrefix'].'_comments';
    }

    /**
    * Builds names of an individuals table from the genealogy name and the type of table
    *
    * @access public
    * @param $genName the name of the genealogy being viewed.
    * @param $suffix the type of the table : individuals, families or comments
    * @return string

    function table( $genName , $suffix ) {
        return $this->environment['dbPrefix'].'_data_'.$genName.'_'.$suffix;
    }
    **/

    /**
    * Returns the name of a module's SQL table
    *
    * @access public
    * @param $moduleName the name of the module
    * @return string
    **/

    function moduleTable( $moduleName ) {
        return $this->environment['dbPrefix'].'_module_'.$moduleName;
    }

    /**
    * Sets the charset used to build the html page
    *
    * @access public
    * @param $charset the charset to be used
    * @return void
    **/

    function setCharset( $charset ) {
        $this->context['charset'] = $charset;
    }

    /**
    * Gets the language from the URL fragment, or from the PHP Gedcom Viewer settings, or from the browser settings
    *
    * @access private
    * @return string
    **
    **/

    function _getLanguage() {
        if ( !empty( $this->context['ul'] ) ) {
            $lang = $this->context['ul'];
        } else if ( !empty( $this->settings['language'] ) ) {
            $lang = $this->settings['language'];
        } else if ( !empty( $_SERVER['HTTP_ACCEPT_LANGUAGE'] ) ) {
            $lang = substr( $_SERVER['HTTP_ACCEPT_LANGUAGE'] , 0 , 2 );
        } else {
            $lang = 'en' ;
        }
        return $lang;
    }

    /**
    * Gets the title of the genealogy being viewed
    *
    * @access public
    * @return void
    **/

    function getGenTitle() {
        if ( $this->context['isGen'] ) {
            if ( !empty( $this->infos['title'] ) ) {
                $this->context['genTitle'] = $this->infos['title'];
            } else {
                $this->context['genTitle'] = GV_GENEALOGY.' #'.$this->context['genID'];
            }
        } else {
            $this->context['genTitle'] = GV_GLOBAL_INDEX;
        }
        $this->context['pageTitle'] = $this->context['genTitle'].' - '.$this->context['mainMenuTitle'];
    }

    /**
    * Sets the main menu title
    *
    * Automatically builds the page title from the main menu title
    * @access public
    * @param $title the name of the page
    * @return void
    **/

    function setMainMenuTitle( $title ) {
        $this->getGenTitle();
        $this->context['mainMenuTitle'] = $title;
        $this->setPageTitle( $this->context['genTitle'].' - '.$this->context['mainMenuTitle'] );
    }

    /**
    * Sets the name of the page
    *
    * @access public
    * @param $title the name of the page
    * @return void
    **/

    function setPageTitle( $title ) {
        $this->context['pageTitle'] = $title;
    }

    /**
    * Gets the theme to be used for the genealogy being viewed
    *
    * @access private
    * @return void
    **/

    function _getTheme() {
        $theme = $this->context['defaultTheme'];
        if ( ( $this->context['isGen'] ) && ( !empty( $this->infos['theme'] ) ) ) {
            $theme = $this->infos['theme'];
        }
        return $theme;
    }

    /**
    * Generates the html code for the PHP Gedcom Viewer logo
    *
    * @access public
    * @return html
    **/

    function displayLogo() {
        if ( $this->context['displayLogo'] ) {
            //return '<a href=\'http://www.poplargedcomviewer.com\' target=\'_blank\'><img src=\''.$this->environment['rootUrl'].'/pictures/logo2.gif\' alt=\'Poplar Gedcom Viewer\' border=\'0\' width=\'166\' height=\'60\'></a><br /><font size=\'1\' color=\'#555555\'>by the PoplarGedcomViewer Team</font>';
            return '<a href=\'http://poplar.sourceforge.net\' target=\'_blank\'><img src=\''.$this->environment['rootUrl'].'/pictures/logo2.gif\' alt=\'Poplar Gedcom Viewer\' border=\'0\' width=\'166\' height=\'60\'></a><br /><font size=\'1\' color=\'#555555\'>by the PoplarGedcomViewer Team</font>';
        } else {
            //return '<a href=\'http://www.poplargedcomviewer.com\' target=\'_blank\'><font size=\'2\' color=\'#555555\' face=\'Arial, Helvetica, sans-serif\'>Poplar Gedcom Viewer</font></a><br /><font size=\'1\' color=\'#555555\'>by Nicolas de Vulpian</font>';
            return '<a href=\'http://poplar.sourceforge.net\' target=\'_blank\'><font size=\'2\' color=\'#555555\' face=\'Arial, Helvetica, sans-serif\'>Poplar Gedcom Viewer</font></a><br /><font size=\'1\' color=\'#555555\'>by the PoplarGedcomViewer Team</font>';
        }
    }

    /**
    * Gets the information necessary to build an URL and stores them in the aLink array
    *
    * @access public
    * @return void
    **/

    function getUrl() {
        $this->_aLink['rootUrl']    = $this->environment['rootUrl'];
        $this->_aLink['genID']      = $this->context['genID'] ;
        $this->_aLink['page']       = $this->context['page'] ;
        $this->_aLink['letter']     = $this->context['letter'] ;
        $this->_aLink['advanced']   = $this->context['advanced'] ;
        $this->_aLink['module']     = $this->context['module'] ;
        $this->_aLink['step']       = $this->context['step'] ;
        $this->_aLink['idSurname']  = $this->context['idSurname'] ;
        $this->_aLink['indID']      = $this->context['indID'] ;
        $this->_aLink['spouseID']   = $this->context['spouseID'] ;
        $this->_aLink['ul']         = $this->context['ul'] ;
    }

    /**
    * Empty the aLink array
    *
    * @access public
    * @return void
    **/

    function emptyUrl() {
        unset( $this->_aLink );
        $this->_aLink['rootUrl']    = $this->environment['rootUrl'];
    }

    /**
    * Set or change a field of the aLink array. Use this to navigate through PHP Gedcom Viewer's pages
    *
    * @access public
    * @param $link the aLink field to be filled
    * @param $value the value of the field
    * @return void
    **/

    function setUrl( $link , $value ) {
        $this->_aLink[$link] = $value;
    }

    /**
    * Builds an url from the aLink array. aLink is filled with getUrl and setUrl methods
    *
    * @access public
    * @return string the URL built from the aLink array with getUrl and setUrl methods
    **/

    function makeUrl() {
        if ( !empty( $this->_aLink['rootUrl'] ) )    { $link = $this->_aLink['rootUrl']; }
        $link .= '/index.php?';
        if ( !empty( $this->_aLink['genID'] ) )      { $link .= '&amp;genID='.$this->_aLink['genID']; }
        if ( !empty( $this->_aLink['page'] ) )       { $link .= '&amp;page='.$this->_aLink['page']; }
        if ( !empty( $this->_aLink['letter'] ) )     { $link .= '&amp;letter='.$this->_aLink['letter']; }
        if ( !empty( $this->_aLink['advanced'] ) )   { $link .= '&amp;advanced='.$this->_aLink['advanced']; }
        if ( !empty( $this->_aLink['module'] ) )     { $link .= '&amp;module='.$this->_aLink['module']; }
        if ( !empty( $this->_aLink['step'] ) )       { $link .= '&amp;step='.$this->_aLink['step']; }
        if ( !empty( $this->_aLink['idSurname'] ) )  { $link .= '&amp;id_surname='.$this->_aLink['idSurname']; }
        if ( !empty( $this->_aLink['indID'] ) )      { $link .= '&amp;id='.$this->_aLink['indID']; }
        if ( !empty( $this->_aLink['spouseID'] ) )   { $link .= '&amp;spouseID='.$this->_aLink['spouseID']; }
        if ( !empty( $this->_aLink['ul'] ) )         { $link .= '&amp;ul='.$this->_aLink['ul']; }
        if ( !empty( $this->_aLink['maxg'] ) )       { $link .= '&amp;maxg='.$this->_aLink['maxg']; }

        $link = str_replace( '?&amp;' , '?' , $link );

        return $link;
    }

    /**
    * Builds an HTML url tag from the aLink array. aLink is filled with getUrl and setUrl methods
    *
    * @acces public
    * @param $text the text used to build the a link
    * @return string the URL built, embedded in <a></a> tags
    **/

    function makeLink( $text ) {
        $url = '<a href=\''.$this->makeUrl().'\'>'.$text.'</a>';

        return $url;
    }

    /**
    * Gets the flags to be displayed and generates the html code
    *
    * @access public
    * @return array a list of image links (actually flags linking to their translations)
    **/

    function getFlags() {
        $dir = opendir( $this->environment['rootPath'].'/lang/' );
        while ( $file = readdir( $dir ) ) {
            if ( $file != '.'
                && $file != '..'
                && ( substr( $file , -4 , 4 ) == '.php' ) ) {
                $file = str_replace( '.php' , '' , $file );

                $this->getUrl();
                $this->setUrl( 'ul' , $file );

                $flags[] = $this->makeLink( '<img src=\''.$this->environment['rootUrl'].'/pictures/flags/'.$file.'.gif\' width=\'24\' height=\'16\' border=\'0\' alt=\''.$file.'\' />' );
            }
        }
        closedir( $dir );
        return $flags;
    }

    /**
    * Builds the main menu links
    *
    * @access public
    * @return array a list of menu links or false if no link
    **/

    function getMainMenu() {
        $this->emptyUrl();                      //there is NO index now
        $this->setUrl( 'ul' , $this->context['ul'] );

        $main[0] = $this->makeLink( GV_GLOBAL_INDEX );
        return $main;
    }

    /**
    **
    * @return void
    **
    **/

    function getContextMenu() {
        if ( $this->context['isGen'] ) {
            $this->emptyUrl();
            $this->setUrl( 'genID' , $this->context['genID'] );
            $this->setUrl( 'ul' , $this->context['ul'] );

            $context[0] = $this->makeLink( GV_HOME );
            return $context;
        } else {
            return false;
        }
    }

    /**
    **
    * @return void
    **
    **/

    function getModules() {
        global $db, $hide;

        $dir = opendir( $this->environment['rootPath'].'/modules/' );

        while ( $file = readdir( $dir ) ) {
            if ( $file != '.' && $file != '..' ){
                if ( file_exists( $this->environment['rootPath'].'/modules/'.$file.'/lang/'.$this->context['language'].'.php' ) ) {
                    include_once( $this->environment['rootPath'].'/modules/'.$file.'/lang/'.$this->context['language'].'.php' );
                } elseif ( file_exists( $this->environment['rootPath'].'/modules/'.$file.'/lang/en.php' ) ) {
                    include_once( $this->environment['rootPath'].'/modules/'.$file.'/lang/en.php' );
                }

                include_once( $this->environment['rootPath'].'/modules/'.$file.'/module_info.php' );
                $modules[$file]          = $module;
                $modules[$file]['file']  = $file;

                $result = $db->Execute( 'SELECT * FROM '.$this->table['modules'].' WHERE name = \''.$file.'\'' );
                if ( $result === false ) die( 'Failed to check if '.$file.' module is activated' );

                if ( !$result->EOF ) {
                    $modules[$file]['enabled']    = true;
                }
            }
        }  
        closedir( $dir );

        $this->modules = $modules;
        return $modules;
    }

    /**
    **
    * @return void
    **
    **/

    function getModule( $moduleName ) {
        global $db, $hide;

        if ( file_exists( $this->environment['rootPath'].'/modules/'.$moduleName.'/lang/'.$this->context['language'].'.php' ) ) {
            include_once( $this->environment['rootPath'].'/modules/'.$moduleName.'/lang/'.$this->context['language'].'.php' );
        } elseif ( file_exists( $this->environment['rootPath'].'/modules/'.$moduleName.'/lang/en.php' ) ) {
            include_once( $this->environment['rootPath'].'/modules/'.$moduleName.'/lang/en.php' );
        }
        include_once( $this->environment['rootPath'].'/modules/'.$moduleName.'/module_info.php' );

        $this->module            = $module;
        $this->module['file']    = $moduleName;
    }

    /**
    **
    * @return void
    **
    **/

    function getModulesMenu() {
        global $db, $hide;

        if ( empty( $this->modules ) ) {
            $this->getModules();
        }
        foreach ( $this->modules as $key=>$module ) {
            if ( $module['enabled']
                && $module['authorized']
                && ( ( $module['pages'] == 'all+' )
                || ( $module['pages'] == 'all-'
                && $this->context['isGen'] )
                || ( !empty( $this->context['page'] )
                && eregi( $this->context['page'] , $module['pages'] ) ) ) ) {
                if ( $module['interface'] ) {
                    $this->getUrl();
                    $this->setUrl( 'page' , 'module' );
                    $this->setUrl( 'module' , $module['file'] );
                //added border jg//
                    $module_links[] = $this->makeLink( '<img src=\''.$this->environment['rootUrl'].'/modules/'.$module['file'].'/pictures/icon.gif\' border="1" width=\'32\' height=\'32\' alt=\''.htmlentities( $module['name'] , ENT_QUOTES ).'\' border=\'0\' />' );
                }

                if ( $module['silent'] ) {
                    include_once( $this->environment['rootPath'].'/modules/'.$module['file'].'/module_silent.php' );
                }
            }
        }
        return $module_links;
    }

    /**
    **
    * @return void
    **
    **/

    function getOwner() {
        if ( !empty( $this->context['ownerName'] )
            && !empty($this->context['ownerEmail'] ) ) {
            $owner['contact'] = fill( GV_OWNER , '<a href=\'mailto:'.$this->context['ownerEmail'].'?subject='.$this->context['genTitle'].'\'>'.stripslashes( $this->context['ownerName'] ).'</a>' );
        }
        if ( empty( $this->context['ownerName'] )
            && !empty( $this->context['ownerEmail'] ) ) {
            $owner['contact'] = fill( GV_OWNER , '<a href=\'mailto:'.$this->context['ownerEmail'].'?subject='.$this->context['genTitle'].'\'>'.$this->context['ownerEmail'].'</a>' );
        }
        if ( !empty( $this->context['ownerName'] )
            && empty( $this->context['ownerEmail'] ) ) {
            $owner['contact'] = fill( GV_OWNER , stripslashes( $this->context['ownerName'] ) );
        }

        if ( !empty( $this->context['ownerWebsite'] ) ) {
            //frames//$owner['website'] = fill( GV_OWNER_WEBSITE , '<a href=\''.$this->context['ownerWebsite'].'\' target=\'_blank\'>'.$this->context['ownerWebsite'].'</a>' );
            //non-frames//
            $owner['website'] = fill( GV_OWNER_WEBSITE , '<a href=\''.$this->context['ownerWebsite'].'\'>'.$this->context['ownerWebsite'].'</a>' );
        }

        return $owner;
    }
}
?>